/*********************************************************************
*               SEGGER MICROCONTROLLER SYSTEME GmbH                  *
*       Solutions for real time microcontroller applications         *
**********************************************************************
*                                                                    *
*       (c) 1995 - 2007  SEGGER Microcontroller Systeme GmbH         *
*                                                                    *
*       www.segger.com     Support: support@segger.com               *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS * Real time operating system for microcontrollers      *
*                                                                    *
*                                                                    *
*       Please note:                                                 *
*                                                                    *
*       Knowledge of this file may under no circumstances            *
*       be used to write a similar product or a real-time            *
*       operating system for in-house use.                           *
*                                                                    *
*       Thank you for your fairness !                                *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS version: 3.32o                                         *
*                                                                    *
**********************************************************************

----------------------------------------------------------------------
File    : RTOSInit_R8C23.c    (for Renesas R8C23 CPU and RENESAS NC30 compiler)
Purpose : Initializes and handles the hardware for embOS as far
          as required by embOS.
          Feel free to modify this file acc. to your target system.
--------  END-OF-HEADER  ---------------------------------------------
*/

#include "RTOS.H"

/*********************************************************************
*
*       Configuration
*
**********************************************************************
*/

/*********************************************************************
*
*       Clock frequency settings
*/
#ifndef   OS_FSYS                   /* CPU main clock frequncy      */
  #define OS_FSYS 20000000L
#endif

#ifndef   OS_PCLK_TIMER             /* Peripheral clock for timer   */
  #define OS_PCLK_TIMER OS_FSYS     /* May vary from CPU clock      */
#endif                              /* depending on CPU             */

#ifndef   OS_PCLK_UART              /* Peripheral clock for UART    */
  #define OS_PCLK_UART OS_FSYS      /* May vary from CPU clock      */
#endif                              /* depending on CPU             */

#define _OS_COUNT_RELOAD  (100)     /* Counter reload value 1..256  */
#define _OS_PRES_RELOAD   (OS_PCLK_TIMER/_OS_COUNT_RELOAD/1000)


#define TRACR             (*(volatile unsigned char* )0x0100)
#define TRAICO            (*(volatile unsigned char* )0x0101)
#define TRAMR             (*(volatile unsigned char* )0x0102)
#define TRAPRE            (*(volatile unsigned char* )0x0103)
#define TRA               (*(volatile unsigned char* )0x0104) 
#define TRAIC             (*(volatile unsigned char* )0x0056) 
#define CM0               (*(volatile unsigned char* )0x0006)
#define CM1               (*(volatile unsigned char* )0x0007)
#define OCD               (*(volatile unsigned char* )0x000C)
#define PRCR              (*(volatile unsigned char* )0x000A)

/*********************************************************************
*
*       UART settings for embOSView
*       If you do not want (or can not due to hardware limitations)
*       to dedicate a UART to embOSView, please define it to be -1
*       Currently the standard code enables UART 1 per default
*       and supports UART0 and UART1
*/
#ifndef   OS_UART
  #define OS_UART 0
#endif
#ifndef   OS_BAUDRATE
  #define OS_BAUDRATE 38400L
#endif

/*********************************************************************
*
*       OS_InitHW
*
*       Initialize the hardware required for the OS to run. This will work
*       on any target hardware, but may have to be tailored a little
*       (regarding the clock frequency). Of course the same holds true
*       if for some reason you choose to use an other timer.
*/
void OS_InitHW(void) {
  OS_IncDI();                          // Initialy disable interrupts
  //
  // Select CPU clock mode, external oscillator used, full speed
  //
  PRCR = 1;                            // switch protection off
  CM1 = 0x28;
  CM0 = 0x08;
  asm("nop");                          // wait a little bit for stable oscillator
  asm("nop");
  asm("nop");
  asm("nop");
  asm("nop");
  OCD = 0;                             // switch to main-clock
  PRCR = 0;                            // switch protection on again
  //
  // enable timer RA
  //
  TRACR  = 0x00;                       // stop counting         
  TRAICO = 0x00;              
  TRAMR  = 0x00;                       // timer mode, clock source = f                    
  TRAPRE = _OS_PRES_RELOAD - 1;        // Prescaler
  TRA    =  _OS_COUNT_RELOAD - 1;      // reload value       
  TRAIC  = 0x01;                       // interrupt level 1
  asm("nop");                          // Using the NOP instruction to keep the program
  asm("nop");                          // waiting until
  asm("nop");                          // the interrupt control register is modified
  TRACR  = 0x01;                       // starts counting
  //
  // Finally initialize UART for embOSView
  //
  OS_COM_Init();                       // Initialize communication to embOSView
  OS_DecRI();                          // Restore interrupt enable condition
}

/*********************************************************************
*
*       OS_Idle()
*
*       Please note:
*       This is basically the "core" of the idle loop
*       This core loop can be changed, but:
*       The idle loop does not have a stack of its own, therefore no
*       functionality should be implemented that relies on the stack
*       to be preserved. However, a simple program loop can be programmed
*       (like toggeling an output or incrementing a counter)
*/
void OS_Idle(void) {      // Idle loop: No task is ready to exec
  for (;;);               // Nothing to do ... wait for an interrupt
}

/*********************************************************************
*
*       OS_GetTime_Cycles()
*
*       This routine is required for profiling info via embOSView
*       or high precision time measurment functions.
*       It returns the system time in clock cycles.
*
*       This function has to be adapted if an other timer is used
*/
OS_U32 OS_GetTime_Cycles(void) {
  OS_U32 Time;
  unsigned int T_Cnt;
  OS_U8  PresValue;
  OS_U8  CntValue;
  OS_U8  CntValuePrev;

  do {
    CntValuePrev = TRA;
    PresValue    = TRAPRE;
    CntValue     = TRA;
  } while (CntValuePrev != CntValue);
  Time = OS_Time;

  if (TRAIC & (1 << 3)) {  /* if timer interrupt pending */
    Time++;
    do {                  // Adjust result
      CntValuePrev = TRA;
      PresValue    = TRAPRE;
      CntValue     = TRA;
    } while (CntValuePrev != CntValue);
  }
  T_Cnt = ((_OS_COUNT_RELOAD - 1 - CntValue) * _OS_PRES_RELOAD) + (_OS_PRES_RELOAD - 1 - PresValue);
  return (OS_PCLK_TIMER / 1000) * Time + T_Cnt;
}

/*********************************************************************
*
*       OS_ConvertCycles2us
*
* Purpose:
*   Convert Cycles into micro seconds.
*
*   If your clock frequency is not a multiple of 1 MHz, you may have to
*   modify this routine in order to get proper diagonstics.
* Please note:
*   This routine is required for profiling or high resolution time
*   measurement. It does not affect operation of embOS.
*/
OS_U32 OS_ConvertCycles2us(OS_U32 Cycles) {
  return Cycles/(OS_PCLK_TIMER/1000000);
}

/*********************************************************************
*
*       OS_ISR_Tick(), embOS timer interrupt handler
*/
#pragma interrupt OS_ISR_Tick
void OS_ISR_Tick(void) {
  OS_EnterNestableInterrupt();
  OS_HandleTick();
  OS_LeaveNestableInterrupt();
}

/*********************************************************************
*
*       Communication for embOSView (UART 0)
*
**********************************************************************
*/

#define BAUDDIVIDE ((OS_PCLK_UART+OS_BAUDRATE*8L)/(OS_BAUDRATE*16L)-1)

/*********************************************************************
*
*       OS_COM_Init()
*/
#if (OS_UART == 0)

#define U0MR       (*(volatile unsigned char* )0x00A0)
#define U0BRG      (*(volatile unsigned char* )0x00A1)
#define U0TBL      (*(volatile unsigned short*)0x00A2)
#define U0C0       (*(volatile unsigned char* )0x00A4)
#define U0C1       (*(volatile unsigned char* )0x00A5)
#define U0RB       (*(volatile unsigned short*)0x00A6)
#define S0TIC      (*(volatile unsigned char* )0x0051)
#define S0RIC      (*(volatile unsigned char* )0x0052)
#define PD1        (*(volatile unsigned char* )0x00E3)

void OS_COM_Init(void) {
  U0MR   = 0x00;           /*  lock Sio, error reset                */
  U0C0   = 0x10;           /*  RTS/CTS disabled, clock divisor 1    */
  U0BRG  = BAUDDIVIDE;     /*  Calculated Baudrate                  */
  U0C1   = 0;              /*  Lock Rx and Tx                       */
  PD1   &= ~(1 << 5);      /*  RxD0, Port set as input              */
  U0MR   = 0x05            /*  8  Data                              */
          +(0 << 4)        /*  0: 1 Stop-Bit (1= 2 Stop-Bits)       */
          +(0 << 5)        /*  X: Parity select                     */
          +(0 << 6)        /*  1: parity enable                     */
          +(0 << 7);       /*  0: no sleep                          */
  U0C1   = 5;              /*  enable reception and transmission    */
  S0RIC  = 1;              /*  Enable UART Rx interrupts            */
  S0TIC  = 1;              /*  Enable UART Tx interrupts            */
}

/*********************************************************************
*
*       OS_COM_Send1() for UART0: Send one byte to embOSView
*
*       NEVER call this function from your application
*/
void OS_COM_Send1(OS_U8 c) {
  U0TBL = c;
}

/*********************************************************************
*
*       OS_ISR_tx() Tx interrupt handler for UART0
*/
#pragma interrupt OS_ISR_tx 
void OS_ISR_tx (void) {
  OS_EnterNestableInterrupt();       /* We will enable interrupts   */
  OS_OnTx();
  OS_LeaveNestableInterrupt();
}

/*********************************************************************
*
*       OS_ISR_rx() Rx interrupt handler for UART0
*
*  Notes:
*    (1) UART receiver buffer register has to be read as 16 bit value
*        according usage notes in datasheet
*/
#pragma interrupt OS_ISR_rx  
void OS_ISR_rx (void) {
  int Data;                     /* Note (1)                    */  
  OS_EnterNestableInterrupt();  /* We will enable interrupts   */
  Data = U0RB;                  /* Note (1)                    */
  if (Data & 0x6000) {          /* Check if errors occurred    */
    U0C1 &= ~(1<<2);            /* disable Rx, then            */
    U0C1 |=  (1<<2);            /* enable Rx, thus clear error */
  } else {
    OS_OnRx(Data);
  }
  OS_LeaveNestableInterrupt();
}

/*********************************************************************
*
*       Communication for embOSView (UART 1)
*
**********************************************************************
*/

#elif (OS_UART == 1)

/* 
*  Notes:
*    UART1 can not be used when Debug unit is used !
*/

#define U1MR       (*(volatile unsigned char* )0x00A8)
#define U1BRG      (*(volatile unsigned char* )0x00A9)
#define U1TBL      (*(volatile unsigned short*)0x00AA)
#define U1C0       (*(volatile unsigned char* )0x00AC)
#define U1C1       (*(volatile unsigned char* )0x00AD)
#define U1RB       (*(volatile unsigned short*)0x00AE)
#define S1TIC      (*(volatile unsigned char* )0x0053)
#define S1RIC      (*(volatile unsigned char* )0x0054)
#define PD1        (*(volatile unsigned char* )0x00E3)
#define PD3        (*(volatile unsigned char* )0x00E7)
#define PD4        (*(volatile unsigned char* )0x00EA)


/*********************************************************************
*
*       OS_COM_Init()
*/
void OS_COM_Init(void) {
  U1MR   = 0x00;           /*  lock Sio, error reset                */
  U1C0   = 0x10;           /*  RTS/CTS disabled, clock divisor 1    */
  U1BRG  = BAUDDIVIDE;     /*  Calculated Baudrate                  */
  U1C1   = 0;              /*  Lock Rx and Tx                       */
  PD4   &= ~(1 << 5);      /*  RxD1, Port set as input              */
  PD3   &= ~(1 << 7);      /*  TxD1, Port set as input              */
  U1MR   = 0x05            /*  8  Data                              */
          +(0 << 4)        /*  0: 1 Stop-Bit (1= 2 Stop-Bits)       */
          +(0 << 5)        /*  X: Parity select                     */
          +(0 << 6)        /*  1: parity enable                     */
          +(0 << 7);       /*  0: no sleep                          */
  U1C1   = 5;              /*  enable reception and transmission    */
  S1RIC  = 1;              /*  Enable UART Rx interrupts            */
  S1TIC  = 1;              /*  Enable UART Tx interrupts            */
}

/*********************************************************************
*
*       OS_COM_Send1() for UART1: Send one byte to embOSView
*
*       NEVER call this function from your application
*/
void OS_COM_Send1(OS_U8 c) {
  U1TBL = c;
}

/*********************************************************************
*
*       OS_ISR_tx() Tx interrupt handler for UART1
*/
#pragma interrupt OS_ISR_tx 
void OS_ISR_tx (void) {
  OS_EnterNestableInterrupt();  /* We will enable interrupts  */
  OS_OnTx();
  OS_LeaveNestableInterrupt();
}

/*********************************************************************
*
*       OS_ISR_rx() Rx interrupt handler for UART1
*
*  Notes:
*    (1) UART receiver buffer register should be read as 16 bit value
*/
#pragma interrupt OS_ISR_rx 
void OS_ISR_rx (void) {
  int Data;                     /* Note (1)                    */  
  OS_EnterNestableInterrupt();  /* We will enable interrupts   */
  Data = U1RB;                  /* Read 16bits !               */
  if (Data & 0x6000) {          /* Check if errors occurred    */
    U1C1 &= ~(1 << 2);          /* disable Rx, then            */
    U1C1 |=  (1 << 2);          /* enable Rx, thus clear error */
  } else {
    OS_OnRx(Data);
  }
  OS_LeaveNestableInterrupt();
}

/*********************************************************************
*
*       Communication for embOSView not selected
*
**********************************************************************
*/

#else  /* No communication routines required, use dummies  */
  void OS_COM_Init(void) {}
  void OS_COM_Send1(OS_U8 c) {
    OS_USEPARA(c);
    OS_COM_ClearTxActive();  /* Free OS transmitter buffer */
  }
#endif /* (OS_UART==0) || (OS_UART==1) */

/*****  EOF  ********************************************************/
